local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local select = select
local GetPowerRegen = GetPowerRegen
local SPELL_POWER_LIGHT_FORCE = SPELL_POWER_LIGHT_FORCE

c.Init(a)

local function addSpell(name, tag, attributes)
	local spell = c.AddSpell(name, tag, attributes)
	spell.NoStopChannel = true
end

local function addOptionalSpell(name, tag, attributes)
	local spell = c.AddOptionalSpell(name, tag, attributes)
	spell.NoStopChannel = true
end

------------------------------------------------------------------------ Common
c.AddOptionalSpell("Legacy of the Emperor", nil, {
	NoRangeCheck = 1,
	CheckFirst = function()
		return c.RaidBuffNeeded(c.FIVE_PERCENT_BUFFS)
	end
})

addOptionalSpell("Touch of Death", nil, {
	CheckFirst = function()
		return c.HasBuff("Death Note")
	end
})

addOptionalSpell("Expel Harm", nil, {
	CheckFirst = function()
		return s.HealthPercent("player") < 90
	end
})

c.AddInterrupt("Spear Hand Strike")

-------------------------------------------------------------------- Brewmaster
addOptionalSpell("Elusive Brew", nil, {
	Buff = "Elusive Brew",
	BuffUnit = "player",
	UseBuffID = true,
	RequireBuff = "Elusive Brew Stacker",
	RequireBuffStack = 7,
	RequireBuffUnit = "player",
	RequireUseBuffID = true,
})

c.AddOptionalSpell("Stance of the Sturdy Ox", nil, {
	Type = "form",
})

c.AddSpell("Blackout Kick", "for Shuffle", {
	CheckFirst = function(z)
		return c.GetBuffDuration("Shuffle") < 2
			and not c.IsAuraPendingFor("Blackout Kick")
	end
})
c.RegisterAura("Blackout Kick", "Shuffle")

c.AddSpell("Keg Smash", nil, {
	Melee = true,
})

c.AddSpell("Keg Smash", "for Dizzying Haze", {
	Melee = true,
	CheckFirst = function()
		return not s.Debuff(c.GetID("Dizzying Haze"), nil, 2 + c.GetBusyTime())
			and not c.IsAuraPendingFor("Keg Smash")
	end
})
c.RegisterAura("Keg Smash", "Dizzying Haze")

addSpell("Jab", "for Brewmaster", {
	CheckFirst = function()
		local regen = select(2, GetPowerRegen())
		if s.PowerMissing() < regen then
			return true -- we will cap within 1 second
		end
		
		if s.PowerMissing("player", SPELL_POWER_LIGHT_FORCE) == 0 then
			return false -- don't waste chi
		end
		
		-- jab unless we can wait and get better use out of expel harm
		return c.GetCooldown("Expel Harm") > 40 / regen
	end
})

addSpell("Expel Harm", "for Brewmaster", {
	CheckFirst = function()
		return s.HealthPercent("player") < 90
	end
})

c.AddOptionalSpell("Purifying Brew", nil, {
	CheckFirst = function()
		return s.Debuff(c.GetID("Moderate Stagger"), "player")
			or s.Debuff(c.GetID("Heavy Stagger"), "player")
			or (s.Debuff(c.GetID("Light Stagger"), "player")
				and s.Power("player", SPELL_POWER_LIGHT_FORCE) > 2)
	end	
})

c.AddOptionalSpell("Guard")

-------------------------------------------------------------------- Windwalker
c.RegisterAura("Tiger Palm", "Tiger Power")

addSpell("Tiger Palm", "for Tiger Power", {
	CheckFirst = function()
		local stack = c.GetBuffStack("Tiger Power")
		local duration = c.GetBuffDuration("Tiger Power")
		if c.IsAuraPendingFor("Tiger Palm") then
			stack = stack + 1
			duration = 20
		end
		return stack < 3 or duration < 1.5
	end
})

addSpell("Tiger Palm", "under Combo Breaker", {
	CheckFirst = function(z)
		return c.HasBuff("Combo Breaker: Tiger Palm")
			and not c.IsCasting("Tiger Palm")
	end
})

addSpell("Rising Sun Kick", "for Debuff", {
	CheckFirst = function()
		return not c.HasBuff("Rising Sun Kick")
			and not c.IsAuraPendingFor("Rising Sun Kick")
	end
})

addOptionalSpell("Expel Harm", "at Cap", {
	CheckFirst = function()
		return s.HealthPercent("player") < 90
			and s.MaxPower("player") - s.Power("player") < 10
	end
})

addSpell("Tigereye Brew", nil, {
	CheckFirst = function()
		return c.GetBuffStack("Tigereye Brew") == 10
	end
})

addSpell("Fists of Fury", nil, {
	Melee = true,
	CheckFirst = function()
		return c.GetBuffDuration("Tiger Power") > 4
			and select(2, GetPowerRegen()) * 4 < s.PowerMissing("player")
	end
})

addSpell("Blackout Kick")

addSpell("Blackout Kick", "under Combo Breaker", {
	CheckFirst = function(z)
		return c.HasBuff("Combo Breaker: Blackout Kick")
			and not c.IsCasting("Blackout Kick")
	end
})

addSpell("Jab")

addSpell("Jab", "at Cap", {
	CheckFirst = function()
		return s.MaxPower("player") - s.Power("player") < 10
	end
})

addSpell("Energizing Brew", nil, {
	CheckFirst = function()
		return s.PowerMissing("player") > 10 + select(2, GetPowerRegen())
	end
})

c.AddOptionalSpell("Stance of the Fierce Tiger", nil, {
	Type = "form",
})
